@@ -31,6 +31,7 @@ G_DEFINE_TYPE (ClockLocation, clock_location, G_TYPE_OBJECT)
 
 typedef struct {
         gchar *name;
+        gchar *city;
 
         SystemTimezone *systz;
 
@@ -68,13 +69,15 @@ static void clock_location_unset_tz (ClockLocation *this);
 static void setup_weather_updates (ClockLocation *loc);
 static void add_to_network_monitor (ClockLocation *loc);
 static void remove_from_network_monitor (ClockLocation *loc);
-static gchar* clock_location_get_valid_weather_code (const gchar *code);
+
+static gchar *clock_location_get_valid_weather_code (const gchar *code);
 
 #define PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), CLOCK_LOCATION_TYPE, ClockLocationPrivate))
 
 ClockLocation *
 clock_location_find_and_ref (GList       *locations,
                              const gchar *name,
+                             const gchar *city,
                              const gchar *timezone,
                              gfloat       latitude,
                              gfloat       longitude,
@@ -90,6 +93,7 @@ clock_location_find_and_ref (GList       *locations,
                     priv->longitude == longitude &&
                     g_strcmp0 (priv->weather_code, code) == 0 &&
                     g_strcmp0 (priv->timezone, timezone) == 0 &&
+                    g_strcmp0 (priv->city, city) == 0 &&
                     g_strcmp0 (priv->name, name) == 0)
                         break;
         }
@@ -101,7 +105,8 @@ clock_location_find_and_ref (GList       *locations,
 }
 
 ClockLocation *
-clock_location_new (const gchar *name, const gchar *timezone,
+clock_location_new (const gchar *name, const gchar *city,
+		    const gchar *timezone,
 		    gfloat latitude, gfloat longitude,
 		    const gchar *code, WeatherPrefs *prefs)
 {
@@ -112,6 +117,7 @@ clock_location_new (const gchar *name, const gchar *timezone,
         priv = PRIVATE (this);
 
         priv->name = g_strdup (name);
+        priv->city = g_strdup (city);
         priv->timezone = g_strdup (timezone);
 
         /* initialize priv->tzname */
@@ -121,7 +127,7 @@ clock_location_new (const gchar *name, const gchar *timezone,
         priv->latitude = latitude;
         priv->longitude = longitude;
 
-        priv->weather_code = clock_location_get_valid_weather_code(code);
+        priv->weather_code = clock_location_get_valid_weather_code (code);
 
 	if (prefs) {
 		priv->temperature_unit = prefs->temperature_unit;
@@ -169,6 +175,7 @@ clock_location_init (ClockLocation *this)
         ClockLocationPrivate *priv = PRIVATE (this);
 
         priv->name = NULL;
+        priv->city = NULL;
 
         priv->systz = system_timezone_new ();
 
@@ -195,6 +202,11 @@ clock_location_finalize (GObject *g_obj)
                 priv->name = NULL;
         }
 
+        if (priv->city) {
+                g_free (priv->city);
+                priv->city = NULL;
+        }
+
         if (priv->systz) {
                 g_object_unref (priv->systz);
                 priv->systz = NULL;
@@ -228,6 +240,17 @@ clock_location_finalize (GObject *g_obj)
         G_OBJECT_CLASS (clock_location_parent_class)->finalize (g_obj);
 }
 
+const gchar *
+clock_location_get_display_name (ClockLocation *loc)
+{
+        ClockLocationPrivate *priv = PRIVATE (loc);
+
+        if (priv->name && priv->name[0])
+                return priv->name;
+        else
+                return priv->city;
+}
+
 const gchar *
 clock_location_get_name (ClockLocation *loc)
 {
@@ -249,6 +272,27 @@ clock_location_set_name (ClockLocation *loc, const gchar *name)
         priv->name = g_strdup (name);
 }
 
+const gchar *
+clock_location_get_city (ClockLocation *loc)
+{
+        ClockLocationPrivate *priv = PRIVATE (loc);
+
+        return priv->city;
+}
+
+void
+clock_location_set_city (ClockLocation *loc, const gchar *city)
+{
+        ClockLocationPrivate *priv = PRIVATE (loc);
+
+        if (priv->city) {
+                g_free (priv->city);
+                priv->city = NULL;
+        }
+
+        priv->city = g_strdup (city);
+}
+
 gchar *
 clock_location_get_timezone (ClockLocation *loc)
 {
@@ -337,7 +381,7 @@ clock_location_set_tz (ClockLocation *this)
         now_t = time (NULL);
         localtime_r (&now_t, &now);
 
-        if (daylight && now.tm_isdst) {
+        if (now.tm_isdst > 0) {
                 clock_location_set_tzname (this, tzname[1]);
         } else {
                 clock_location_set_tzname (this, tzname[0]);
@@ -533,8 +577,7 @@ clock_location_make_current (ClockLocation *loc,
 
         filename = g_build_filename (SYSTEM_ZONEINFODIR, priv->timezone, NULL);
         set_system_timezone_async (filename,
-				   transient_parent_xid,
-                                   (GFunc)make_current_cb, 
+                                   (GFunc)make_current_cb,
 				   mcdata,
                                    free_make_current_data);
         g_free (filename);
@@ -818,11 +861,12 @@ setup_weather_updates (ClockLocation *loc)
 		priv->weather_timeout = 0;
 	}
 
-	if (!priv->weather_code || strcmp (priv->weather_code, WEATHER_EMPTY_CODE) == 0)
+	if (!priv->weather_code ||
+	    strcmp (priv->weather_code, WEATHER_EMPTY_CODE) == 0)
 		return;
 
 	dms = rad2dms (priv->latitude, priv->longitude);
-	wl = weather_location_new (priv->name, priv->weather_code,
+	wl = weather_location_new (priv->city, priv->weather_code,
 				   NULL, NULL, dms, NULL, NULL);
 
 	priv->weather_info =
