@@ -27,12 +27,12 @@
 #include "panel-properties-dialog.h"
 
 #include <string.h>
-#include <glade/glade-xml.h>
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-icon-entry.h>
 
 #include <libpanel-util/panel-error.h>
 #include <libpanel-util/panel-glib.h>
+#include <libpanel-util/panel-gtk.h>
+#include <libpanel-util/panel-icon-chooser.h>
 #include <libpanel-util/panel-show.h>
 
 #include "nothing.h"
@@ -40,6 +40,7 @@
 #include "panel-gconf.h"
 #include "panel-util.h"
 #include "panel-globals.h"
+#include "panel-icon-names.h"
 
 typedef struct {
 	PanelToplevel *toplevel;
@@ -55,7 +56,7 @@ typedef struct {
 	GtkWidget     *size_label;
 	GtkWidget     *size_label_pixels;
 	GtkWidget     *icon_align;
-	GtkWidget     *icon_entry;
+	GtkWidget     *icon_chooser;
 	GtkWidget     *icon_label;
   	GtkWidget     *expand_toggle;
 	GtkWidget     *autohide_toggle;
@@ -79,9 +80,6 @@ typedef struct {
 	guint          toplevel_notify;
 	guint          background_notify;
 
-	/* The theme directory of the icon, see bug #119209 */
-	char          *icon_theme_dir;
-
 	/* FIXME: This is a workaround for GTK+ bug #327243 */
 	int            selection_emitted;
 } PanelPropertiesDialog;
@@ -107,9 +105,6 @@ panel_properties_dialog_free (PanelPropertiesDialog *dialog)
 		gtk_widget_destroy (dialog->properties_dialog);
 	dialog->properties_dialog = NULL;
 
-	g_free (dialog->icon_theme_dir);
-	dialog->icon_theme_dir = NULL;
-
 	g_free (dialog);
 }
 
@@ -155,7 +150,7 @@ panel_properties_dialog_orientation_changed (PanelPropertiesDialog *dialog,
 
 static void
 panel_properties_dialog_setup_orientation_combo (PanelPropertiesDialog *dialog,
-						 GladeXML              *gui)
+						 GtkBuilder            *gui)
 {
 	PanelOrientation  orientation;
 	GtkListStore     *model;
@@ -163,9 +158,9 @@ panel_properties_dialog_setup_orientation_combo (PanelPropertiesDialog *dialog,
 	GtkCellRenderer  *renderer;
 	int               i;
 
-	dialog->orientation_combo = glade_xml_get_widget (gui, "orientation_combo");
+	dialog->orientation_combo = PANEL_GTK_BUILDER_GET (gui, "orientation_combo");
 	g_return_if_fail (dialog->orientation_combo != NULL);
-	dialog->orientation_label = glade_xml_get_widget (gui, "orientation_label");
+	dialog->orientation_label = PANEL_GTK_BUILDER_GET (gui, "orientation_label");
 	g_return_if_fail (dialog->orientation_label != NULL);
 
 	orientation = panel_profile_get_toplevel_orientation (dialog->toplevel);
@@ -215,15 +210,15 @@ panel_properties_dialog_size_changed (PanelPropertiesDialog *dialog,
 
 static void
 panel_properties_dialog_setup_size_spin (PanelPropertiesDialog *dialog,
-					 GladeXML              *gui)
+					 GtkBuilder            *gui)
 {
-	dialog->size_widgets = glade_xml_get_widget (gui, "size_widgets");
+	dialog->size_widgets = PANEL_GTK_BUILDER_GET (gui, "size_widgets");
 	g_return_if_fail (dialog->size_widgets != NULL);
-	dialog->size_spin = glade_xml_get_widget (gui, "size_spin");
+	dialog->size_spin = PANEL_GTK_BUILDER_GET (gui, "size_spin");
 	g_return_if_fail (dialog->size_spin != NULL);
-	dialog->size_label = glade_xml_get_widget (gui, "size_label");
+	dialog->size_label = PANEL_GTK_BUILDER_GET (gui, "size_label");
 	g_return_if_fail (dialog->size_label != NULL);
-	dialog->size_label_pixels = glade_xml_get_widget (gui, "size_label_pixels");
+	dialog->size_label_pixels = PANEL_GTK_BUILDER_GET (gui, "size_label_pixels");
 	g_return_if_fail (dialog->size_label_pixels != NULL);
 
 	gtk_spin_button_set_range (GTK_SPIN_BUTTON (dialog->size_spin),
@@ -246,80 +241,42 @@ panel_properties_dialog_setup_size_spin (PanelPropertiesDialog *dialog,
 }
 
 static void
-panel_properties_dialog_icon_changed (PanelPropertiesDialog *dialog,
-				      GnomeIconEntry        *entry)
+panel_properties_dialog_icon_changed (PanelIconChooser      *chooser,
+				      const char            *icon,
+				      PanelPropertiesDialog *dialog)
 {
-	const char *icon = NULL;
-        char       *freeme = NULL;
-
-        icon = gnome_icon_entry_get_filename (entry);
-
-        /* Strip dir from the icon path if in the icon
-         * theme directory.  See bug #119209
-         */
-        if (icon && g_path_is_absolute (icon)) {
-                char *dir;
-
-                dir = g_path_get_dirname (icon);
-
-                if (dir && dialog->icon_theme_dir && strcmp (dir, dialog->icon_theme_dir) == 0)
-                        icon = freeme = g_path_get_basename (icon);
-
-                g_free (dir);
-        }
-
         panel_profile_set_attached_custom_icon (dialog->toplevel, icon);
-
-        g_free (freeme);
 }
 
 static void
-panel_properties_dialog_setup_icon_entry (PanelPropertiesDialog *dialog,
-					  GladeXML              *gui)
+panel_properties_dialog_setup_icon_chooser (PanelPropertiesDialog *dialog,
+					    GtkBuilder            *gui)
 {
 	char *custom_icon;
 
-	dialog->icon_align = glade_xml_get_widget (gui, "icon_align");
+	dialog->icon_align = PANEL_GTK_BUILDER_GET (gui, "icon_align");
 	g_return_if_fail (dialog->icon_align != NULL);
 
-	dialog->icon_entry = glade_xml_get_widget (gui, "icon_entry");
-	g_return_if_fail (dialog->icon_entry != NULL);
+	dialog->icon_chooser = panel_icon_chooser_new (NULL);
+	panel_icon_chooser_set_fallback_icon_name (PANEL_ICON_CHOOSER (dialog->icon_chooser),
+						   PANEL_ICON_DRAWER);
+	gtk_widget_show (dialog->icon_chooser);
+	gtk_container_add (GTK_CONTAINER (dialog->icon_align),
+			   dialog->icon_chooser);
 
-	dialog->icon_label = glade_xml_get_widget (gui, "icon_label");
+	dialog->icon_label = PANEL_GTK_BUILDER_GET (gui, "icon_label");
 	g_return_if_fail (dialog->icon_label != NULL);
 
-	dialog->icon_theme_dir = NULL;
 	custom_icon = panel_profile_get_attached_custom_icon (dialog->toplevel);
-	if (custom_icon != NULL &&
-	    ! g_path_is_absolute (custom_icon)) {
-		/* if the icon is not absolute path name it comes from the
-		   theme, and as such we wish to store the theme directory
-		   where it comes from.  See bug #119209 */
-		char *icon;
-		
-		icon = panel_find_icon (gtk_icon_theme_get_default (),
-					custom_icon, 48);
-		if (icon != NULL)
-			dialog->icon_theme_dir = g_path_get_dirname (icon);
-
-		g_free (custom_icon);
-		custom_icon = icon;
-	}
-	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (dialog->icon_entry), custom_icon);
+	panel_icon_chooser_set_icon (PANEL_ICON_CHOOSER (dialog->icon_chooser),
+				     custom_icon);
 	g_free (custom_icon);
 
-	if (dialog->icon_theme_dir == NULL) {
-		/* use the default pixmap directory as the standard icon_theme_dir,
-		 * since the standard directory is themed */
-		g_object_get (G_OBJECT (dialog->icon_entry), "pixmap_subdir",
-			      &(dialog->icon_theme_dir), NULL);
-	}
-
-	g_signal_connect_swapped (dialog->icon_entry, "changed",
-				  G_CALLBACK (panel_properties_dialog_icon_changed), dialog);
+	g_signal_connect (dialog->icon_chooser, "changed",
+			  G_CALLBACK (panel_properties_dialog_icon_changed), dialog);
 
 	if (!panel_profile_is_writable_attached_custom_icon (dialog->toplevel)) {
-		gtk_widget_set_sensitive (dialog->icon_entry, FALSE);
+		gtk_widget_set_sensitive (dialog->icon_chooser, FALSE);
 		gtk_widget_set_sensitive (dialog->icon_label, FALSE);
 		gtk_widget_show (dialog->writability_warn_general);
 	}
@@ -337,9 +294,9 @@ panel_properties_dialog_setup_icon_entry (PanelPropertiesDialog *dialog,
 	}                                                                                         \
 	static void                                                                               \
 	panel_properties_dialog_setup_##n (PanelPropertiesDialog *dialog,                         \
-					   GladeXML              *gui)                            \
+					   GtkBuilder            *gui)                            \
 	{                                                                                         \
-		dialog->n = glade_xml_get_widget (gui, wid);                                      \
+		dialog->n = PANEL_GTK_BUILDER_GET (gui, wid);                                      \
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->n),                      \
 					      panel_profile_get_toplevel_##p (dialog->toplevel)); \
 		g_signal_connect_swapped (dialog->n, "toggled",                                   \
@@ -369,13 +326,13 @@ panel_properties_dialog_color_changed (PanelPropertiesDialog *dialog,
 
 static void
 panel_properties_dialog_setup_color_button (PanelPropertiesDialog *dialog,
-					    GladeXML              *gui)
+					    GtkBuilder            *gui)
 {
 	PanelColor color;
 
-	dialog->color_button = glade_xml_get_widget (gui, "color_button");
+	dialog->color_button = PANEL_GTK_BUILDER_GET (gui, "color_button");
 	g_return_if_fail (dialog->color_button != NULL);
-	dialog->color_label = glade_xml_get_widget (gui, "color_label");
+	dialog->color_label = PANEL_GTK_BUILDER_GET (gui, "color_label");
 	g_return_if_fail (dialog->color_label != NULL);
 
 	panel_profile_get_background_color (dialog->toplevel, &color);
@@ -412,54 +369,14 @@ panel_properties_dialog_image_changed (PanelPropertiesDialog *dialog)
 	g_free (image);
 }
 
-static void
-panel_properties_dialog_chooser_preview_update (GtkFileChooser *file_chooser,
-						gpointer data)
-{
-	GtkWidget *preview;
-	char      *filename;
-	GdkPixbuf *pixbuf;
-	gboolean   have_preview;
-
-	preview = GTK_WIDGET (data);
-	filename = gtk_file_chooser_get_preview_filename (file_chooser);
-
-	if (filename == NULL)
-		return;
-
-	pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 128, 128, NULL);
-	have_preview = (pixbuf != NULL);
-	g_free (filename);
-
-	gtk_image_set_from_pixbuf (GTK_IMAGE (preview), pixbuf);
-	if (pixbuf)
-		g_object_unref (pixbuf);
-
-	gtk_file_chooser_set_preview_widget_active (file_chooser,
-						    have_preview);
-}
-
 static void
 panel_properties_dialog_setup_image_chooser (PanelPropertiesDialog *dialog,
-					     GladeXML              *gui)
+					     GtkBuilder            *gui)
 {
-	GtkFileFilter *filter;
-	GtkWidget     *chooser_preview;
-	char          *image;
+	char *image;
 
-	dialog->image_chooser = glade_xml_get_widget (gui, "image_chooser");
-
-	filter = gtk_file_filter_new ();
-	gtk_file_filter_add_pixbuf_formats (filter);
-	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog->image_chooser),
-				     filter);
-
-	chooser_preview = gtk_image_new ();
-	gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (dialog->image_chooser),
-					     chooser_preview);
-	g_signal_connect (dialog->image_chooser, "update-preview",
-			  G_CALLBACK (panel_properties_dialog_chooser_preview_update),
-			  chooser_preview);
+	dialog->image_chooser = PANEL_GTK_BUILDER_GET (gui, "image_chooser");
+	panel_gtk_file_chooser_add_image_preview (GTK_FILE_CHOOSER (dialog->image_chooser));
 
 	image = panel_profile_get_background_image (dialog->toplevel);
 
@@ -503,16 +420,16 @@ panel_properties_dialog_opacity_changed (PanelPropertiesDialog *dialog)
 
 static void
 panel_properties_dialog_setup_opacity_scale (PanelPropertiesDialog *dialog,
-					     GladeXML              *gui)
+					     GtkBuilder            *gui)
 {
 	guint16 opacity;
 	gdouble percentage;
 
-	dialog->opacity_scale = glade_xml_get_widget (gui, "opacity_scale");
+	dialog->opacity_scale = PANEL_GTK_BUILDER_GET (gui, "opacity_scale");
 	g_return_if_fail (dialog->opacity_scale != NULL);
-	dialog->opacity_label = glade_xml_get_widget (gui, "opacity_label");
+	dialog->opacity_label = PANEL_GTK_BUILDER_GET (gui, "opacity_label");
 	g_return_if_fail (dialog->opacity_label != NULL);
-	dialog->opacity_legend = glade_xml_get_widget (gui, "opacity_legend");
+	dialog->opacity_legend = PANEL_GTK_BUILDER_GET (gui, "opacity_legend");
 	g_return_if_fail (dialog->opacity_legend != NULL);
 
 	opacity = panel_profile_get_background_opacity (dialog->toplevel);
@@ -568,16 +485,16 @@ panel_properties_dialog_background_toggled (PanelPropertiesDialog *dialog,
 				
 static void
 panel_properties_dialog_setup_background_radios (PanelPropertiesDialog *dialog,
-						 GladeXML              *gui)
+						 GtkBuilder            *gui)
 {
 	PanelBackgroundType  background_type;
 	GtkWidget           *active_radio;
 
-	dialog->default_radio     = glade_xml_get_widget (gui, "default_radio");
-	dialog->color_radio       = glade_xml_get_widget (gui, "color_radio");
-	dialog->image_radio       = glade_xml_get_widget (gui, "image_radio");
-	dialog->color_widgets     = glade_xml_get_widget (gui, "color_widgets");
-	dialog->image_widgets     = glade_xml_get_widget (gui, "image_widgets");
+	dialog->default_radio     = PANEL_GTK_BUILDER_GET (gui, "default_radio");
+	dialog->color_radio       = PANEL_GTK_BUILDER_GET (gui, "color_radio");
+	dialog->image_radio       = PANEL_GTK_BUILDER_GET (gui, "image_radio");
+	dialog->color_widgets     = PANEL_GTK_BUILDER_GET (gui, "color_widgets");
+	dialog->image_widgets     = PANEL_GTK_BUILDER_GET (gui, "image_widgets");
 
 	background_type = panel_profile_get_background_type (dialog->toplevel);
 	switch (background_type) {
@@ -912,7 +829,7 @@ panel_properties_dialog_remove_orientation_combo (PanelPropertiesDialog *dialog)
 }
 
 static void
-panel_properties_dialog_remove_icon_entry (PanelPropertiesDialog *dialog)
+panel_properties_dialog_remove_icon_chooser (PanelPropertiesDialog *dialog)
 {
 	GtkContainer *container = GTK_CONTAINER (dialog->general_table);
 
@@ -921,7 +838,7 @@ panel_properties_dialog_remove_icon_entry (PanelPropertiesDialog *dialog)
 
 	dialog->icon_label = NULL;
 	dialog->icon_align = NULL;
-	dialog->icon_entry = NULL;
+	dialog->icon_chooser = NULL;
 
 	gtk_table_resize (GTK_TABLE (dialog->general_table), 3, 2);
 }
@@ -943,7 +860,7 @@ panel_properties_dialog_update_for_attached (PanelPropertiesDialog *dialog,
 					     gboolean               attached)
 {
 	if (!attached)
-		panel_properties_dialog_remove_icon_entry (dialog);
+		panel_properties_dialog_remove_icon_chooser (dialog);
 	else {
 		gtk_window_set_title (GTK_WINDOW (dialog->properties_dialog),
 				      _("Drawer Properties"));
@@ -954,7 +871,7 @@ panel_properties_dialog_update_for_attached (PanelPropertiesDialog *dialog,
 
 static PanelPropertiesDialog *
 panel_properties_dialog_new (PanelToplevel *toplevel,
-			     GladeXML      *gui)
+			     GtkBuilder    *gui)
 {
 	PanelPropertiesDialog *dialog;
 
@@ -967,7 +884,7 @@ panel_properties_dialog_new (PanelToplevel *toplevel,
 
 	dialog->toplevel = toplevel;
 
-	dialog->properties_dialog = glade_xml_get_widget (gui, "panel_properties_dialog");
+	dialog->properties_dialog = PANEL_GTK_BUILDER_GET (gui, "panel_properties_dialog");
 	g_signal_connect_swapped (dialog->properties_dialog, "response",
 				  G_CALLBACK (panel_properties_dialog_response), dialog);
 	g_signal_connect_swapped (dialog->properties_dialog, "destroy",
@@ -976,15 +893,15 @@ panel_properties_dialog_new (PanelToplevel *toplevel,
 	gtk_window_set_screen (GTK_WINDOW (dialog->properties_dialog),
 			       gtk_window_get_screen (GTK_WINDOW (toplevel)));
 
-	dialog->writability_warn_general = glade_xml_get_widget (gui, "writability_warn_general");
-	dialog->writability_warn_background = glade_xml_get_widget (gui, "writability_warn_background");
+	dialog->writability_warn_general = PANEL_GTK_BUILDER_GET (gui, "writability_warn_general");
+	dialog->writability_warn_background = PANEL_GTK_BUILDER_GET (gui, "writability_warn_background");
 
-	dialog->general_vbox  = glade_xml_get_widget (gui, "general_vbox");
-	dialog->general_table = glade_xml_get_widget (gui, "general_table");
+	dialog->general_vbox  = PANEL_GTK_BUILDER_GET (gui, "general_vbox");
+	dialog->general_table = PANEL_GTK_BUILDER_GET (gui, "general_table");
 
 	panel_properties_dialog_setup_orientation_combo  (dialog, gui);
 	panel_properties_dialog_setup_size_spin          (dialog, gui);
-	panel_properties_dialog_setup_icon_entry         (dialog, gui);
+	panel_properties_dialog_setup_icon_chooser       (dialog, gui);
 	panel_properties_dialog_setup_expand_toggle      (dialog, gui);
 	panel_properties_dialog_setup_autohide_toggle    (dialog, gui);
 	panel_properties_dialog_setup_hidebuttons_toggle (dialog, gui);
@@ -1024,7 +941,7 @@ panel_properties_dialog_new (PanelToplevel *toplevel,
 
 	g_signal_connect (dialog->properties_dialog, "event",
 			  G_CALLBACK (config_event),
-			  glade_xml_get_widget (gui, "notebook"));
+			  PANEL_GTK_BUILDER_GET (gui, "notebook"));
 
 	gtk_widget_show (dialog->properties_dialog);
 
@@ -1035,7 +952,8 @@ void
 panel_properties_dialog_present (PanelToplevel *toplevel)
 {
 	PanelPropertiesDialog *dialog;
-	GladeXML              *gui;
+	GtkBuilder            *gui;
+	GError                *error;
 
 	if (!panel_properties_dialog_quark)
 		panel_properties_dialog_quark =
@@ -1049,19 +967,29 @@ panel_properties_dialog_present (PanelToplevel *toplevel)
 		return;
 	}
 
-	gui = glade_xml_new (GLADEDIR "/panel-properties-dialog.glade",
-			     "panel_properties_dialog",
-			     NULL);
-	if (gui == NULL) {
+	gui = gtk_builder_new ();
+	gtk_builder_set_translation_domain (gui, GETTEXT_PACKAGE);
+
+	error = NULL;
+	gtk_builder_add_from_file (gui,
+				   BUILDERDIR "/panel-properties-dialog.ui",
+				   &error);
+
+        if (error) {
 		char *secondary;
-		secondary = g_strdup_printf (_("Unable to load file '%s'."),
-					     GLADEDIR"/panel-properties-dialog.glade");
+
+		secondary = g_strdup_printf (_("Unable to load file '%s': %s."),
+					     BUILDERDIR"/panel-properties-dialog.ui",
+					     error->message);
 		panel_error_dialog (GTK_WINDOW (toplevel),
 				    gtk_window_get_screen (GTK_WINDOW (toplevel)),
 				    "cannot_display_properties_dialog", TRUE,
 				    _("Could not display properties dialog"),
 				    secondary);
 		g_free (secondary);
+		g_error_free (error);
+		g_object_unref (gui);
+
 		return;
 	}
 
